package org.runetracker;

import java.awt.event.ActionEvent;
import java.util.Date;
import java.util.TimeZone;

import org.runetracker.actions.UpdateNow;

/**
 * A <tt>Runnable</tt> object that can be placed in a thread for it to update
 * the application's default RuneScape user on RuneTracker automatically.
 * <p>
 * Due to the possibility of modifying the update time while this
 * <tt>Runnable</tt> is running, it continuously sleeps for one second instead
 * of sleeping until the next update.
 * 
 * @author Cynthia of irc.rscript.org
 */
public class AutomaticUpdater implements Runnable {
	public void runOld() {
		long lastUpdateHalfDay = -1;
		while (true) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// don't care
			}
			// If it was not updated during this program run,
			// or it was updated yesterday,
			// or the setting to update twice a day is enabled and it was
			// updated yester-halfday,
			// -and- it must be updated automatically, update it.

			// When this thread starts up, if the local time is equal to, or
			// later than, the automatic update time, an automatic update will
			// be performed immediately. If you do not want this behavior,
			// change 'long lastUpdateHalfDay = -1;' above to
			// 'long lastUpdateHalfDay = new Date().getTime() / (12 * 60 * 60 *
			// 1000);'
			long localTime = (new Date().getTime() % (24 * 60 * 60 * 1000) + TimeZone.getDefault().getOffset(new Date().getTime())) / 1000;
			if (localTime < 0)
				localTime += 24 * 60 * 60;
			else if (localTime >= 24 * 60 * 60)
				localTime -= 24 * 60 * 60;
			if (((lastUpdateHalfDay == -1 || lastUpdateHalfDay < new Date().getTime() / (12 * 60 * 60 * 1000)) && SettingsManager.getInstance().getSettings().isAutoUpdate() && SettingsManager.getInstance().getSettings().isAutoUpdateTwiceDaily() && (localTime % (12 * 60 * 60)) >= (SettingsManager.getInstance().getSettings().getAutoUpdateTime() % (12 * 60 * 60)))
					|| ((lastUpdateHalfDay == -1 || lastUpdateHalfDay / 2 < new Date().getTime() / (24 * 60 * 60 * 1000)) && SettingsManager.getInstance().getSettings().isAutoUpdate() && localTime >= SettingsManager.getInstance().getSettings().getAutoUpdateTime())) {
				lastUpdateHalfDay = new Date().getTime() / (12 * 60 * 60 * 1000);
				new UpdateNow().actionPerformed(new ActionEvent(this, 0, ""));
				System.err.println("Updated stats at " + new Date());
			}
		}
	}

	@Override
	public void run() {
		long lastUpdateHalfDayLocal = -1;
		while (true) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// don't care
			}
			// If it was not updated during this program run,
			// or it was updated yesterday (in local time),
			// or the setting to update twice a day is enabled and it was
			// updated yester-halfday (in local time),
			// -and- it must be updated automatically, update it.

			// When this thread starts up, if the local time is equal to, or
			// later than, the automatic update time, an automatic update will
			// be performed immediately. If you do not want this behavior,
			// change 'long lastUpdateHalfDayLocal = -1;' above so that it reads
			// like the calculation to assign to lastUpdateHalfDayLocal below.
			long localDate = new Date().getTime();
			localDate += TimeZone.getDefault().getOffset(localDate);
			long localTime = (localDate % (24 * 60 * 60 * 1000)) / 1000;
			if (((lastUpdateHalfDayLocal == -1 || lastUpdateHalfDayLocal < localDate / (12 * 60 * 60 * 1000)) && SettingsManager.getInstance().getSettings().isAutoUpdate() && SettingsManager.getInstance().getSettings().isAutoUpdateTwiceDaily() && (localTime % (12 * 60 * 60)) >= (SettingsManager.getInstance().getSettings().getAutoUpdateTime() % (12 * 60 * 60)))
					|| ((lastUpdateHalfDayLocal == -1 || lastUpdateHalfDayLocal / 2 < localDate / (24 * 60 * 60 * 1000)) && SettingsManager.getInstance().getSettings().isAutoUpdate() && localTime >= SettingsManager.getInstance().getSettings().getAutoUpdateTime())) {
				lastUpdateHalfDayLocal = localDate / (12 * 60 * 60 * 1000);
				new UpdateNow().actionPerformed(new ActionEvent(this, 0, ""));
				System.err.println("Updated stats at " + new Date());
			}
		}
	}
}
